/****************************************************************************
*
*     Copyright (c) 2005 Dave Hylands
*           All Rights Reserved
*
*   Permission is granted to any individual or institution to use, copy, or
*   redistribute this software so long as it is not sold for profit, and that
*   this copyright notice is retained.
*
****************************************************************************/
/**
*
*   @file   Log.cpp
*
*   @brief  This file contains the implementation of the logging functions.
*
****************************************************************************/

// ---- Include Files -------------------------------------------------------

#include <stdio.h>

#include "Log.h"

// ---- Public Variables ----------------------------------------------------
// ---- Private Constants and Types -----------------------------------------

#if defined( AVR )

#undef  Log
#undef  LogError
#undef  vLog

#define Log         Log_P
#define LogError    LogError_P
#define vLog        vLog_P

#define char        prog_char

#else

int gVerbose = false;
int gDebug = false;

#endif

// ---- Private Variables ---------------------------------------------------

FILE   *gLogFs = NULL;

// ---- Private Function Prototypes -----------------------------------------
// ---- Functions -----------------------------------------------------------

/**
 * @addtogroup Log
 * @{
 */

//***************************************************************************
/**
*   Sets the logging stream
*/

void LogInit( FILE *logFs )
{
    gLogFs = logFs;

} // LogInit

//***************************************************************************
/**
*   Logs a string using printf like semantics
*/

void Log
(
    const char *fmt,    ///< printf style format specifier
    ...                 ///< variable list of arguments
)
{
    va_list args;

    va_start( args, fmt );
    vLog( fmt, args );
    va_end( args );
}

//***************************************************************************
/**
*   Logs a string using printf like semantics
*/

void vLog
(
    const char *fmt,    ///< printf style format specified
    va_list     args    ///< variable list of arguments
)
{
    // For now we call printf directly. A better way would be to install
    // a callback which does the real work

#if defined( AVR )
    vfprintf_P( gLogFs, fmt, args );
#else
    vfprintf( gLogFs, fmt, args );

    fflush( gLogFs );
#endif
}

/***************************************************************************/
/**
*   Logs an error
*/

void LogError
(
    const char *fmt,    ///< printf style format specifier
    ...                 ///< variable list of arguments
)
{
    va_list args;

#if defined( AVR )
    Log_P( PSTR( "ERROR: " ));
#else
    Log( "ERROR: " );
#endif

    va_start( args, fmt );
    vLog( fmt, args );
    va_end( args );

} // LogError

/** @} */

