###########################################################################
#
#	mkRules-gcc.mk
#
#	This file contains rules for building using a gcc toolchain
#
###########################################################################

#--------------------------------------------------------------------------
#
#	CFLAGS 		= flags used for C compiler
#	CXXFLAGS	= flags for the C++ compiler
#	CPPFLAGS	= flags for the C preprocessor (-D, -I)
#

PREPROCESS.c = $(CC) $(CPPFLAGS) $(TARGET_ARCH) -E -Wp,-C,-dD,-dI

#--------------------------------------------------------------------------
#
# Since we can generate the dependencies as part of the compilation, we
# do that rather than generate the dependency files separately.
		   
#--------------------------------------------------------------------------
#
# Compile C source files
#
#   COMPILE.c = $(CC) $(CFLAGS) $(CPPFLAGS) $(TARGET_ARCH) -c
#

$(MK_OBJ_DIR)/%.o : %.c
	$(ECHO) "Compiling $< ..."
	$(Q)$(COMPILE.c) $(DEP_OUTPUT_OPTION) $(OUTPUT_OPTION) $<

#--------------------------------------------------------------------------
#
# Compile C++ source files
#
#   COMPILE.cc = $(CXX) $(CXXFLAGS) $(CPPFLAGS) $(TARGET_ARCH) -c
#

$(MK_OBJ_DIR)/%.o : %.cpp
	$(ECHO) "Compiling $< ..."
	$(Q)$(COMPILE.cc) $(DEP_OUTPUT_OPTION) $(OUTPUT_OPTION) $<

#--------------------------------------------------------------------------
#
# Generate Preprocessed files from C/C++ source (useful for debugging)
#

%.pp : %.c FORCE
	$(ECHO) "Preprocessing $< ..."
	$(Q)$(PREPROCESS.c) $< > $@

%.pp : %.cpp FORCE
	$(ECHO) "Preprocessing $< ..."
	$(Q)$(PREPROCESS.c) $< > $@

#--------------------------------------------------------------------------
#
# Make the object directory
#

mkdir_obj: $(MK_OBJ_DIR)

$(MK_OBJ_DIR):
	$(ECHO) "Makiing directory $(MK_OBJ_DIR) ..."
	$(Q)$(MKDIR) -p $(MK_OBJ_DIR)

.PHONY: mkdir_obj

#--------------------------------------------------------------------------
#
# Make the executable directory
#

mkdir_exe: $(MK_EXE_DIR)

$(MK_EXE_DIR):
	$(ECHO) "Makiing directory $(MK_EXE_DIR) ..."
	$(Q)$(MKDIR) -p $(MK_EXE_DIR)

.PHONY: mkdir_exe

#--------------------------------------------------------------------------
#
# Create the project archive file
#

ifneq ($(MK_LIB_BASE),)

$(MK_LIB_BASE) : mkdir_obj 	$(call MK_LIB_NAME, $(MK_LIB_BASE))

$(call MK_LIB_NAME, $(MK_LIB_BASE)) : $(MK_OBJ_FILES)
	$(ECHO) "Creating Archive $@ ..."
	$(Q)$(MKDIR) -p $(@D)
	$(Q)$(RM) -f $@
	$(Q)$(AR) -rcs $@ $^
	$(ECHO)

clean_lib : clean_lib_$(MK_LIB_BASE)
clean_lib_$(MK_LIB_BASE) :
	$(ECHO) "Removing $(MK_LIB_BASE) library ..."
	$(Q)$(RM) -f $(MK_LIB_NAME)

.PHONY: clean_lib_$(MK_LIB_BASE)

endif

#--------------------------------------------------------------------------
#
# Create a hex file from an elf file
#

ifneq ($(MK_HEX_TARGET),)

$(MK_HEX_TARGET): mkdir_exe mkdir_obj $(call MK_HEX_NAME, $(MK_HEX_TARGET))

.PHONY: $(MK_HEX_TARGET)

$(call MK_HEX_NAME, %) : $(call MK_EXE_NAME, %)
	$(ECHO) "Creating $@ ..."
	$(Q)$(OBJCOPY) -j .text -j .data -O ihex $< $@
	$(ECHO)

MK_LINK_TARGET += $(MK_HEX_TARGET)
endif


#--------------------------------------------------------------------------
#
# Create simple executables
#

ifneq ($(MK_EXE_TARGET),)

$(MK_EXE_TARGET): mkdir_exe mkdir_obj $(call MK_EXE_NAME, $(MK_EXE_TARGET))

.PHONY: $(MK_EXE_TARGET)

MK_LINK_TARGET += $(MK_EXE_TARGET)

endif

ifneq ($(MK_LINK_TARGET),)

ifneq ($(filter %.cpp,$(MK_SRC_FILES)),)
LINK.o = $(CXX) $(LDFLAGS) $(TARGET_ARCH)
endif

$(call MK_EXE_NAME, %) : $(MK_OBJ_DIR)/%.o
	$(ECHO) "Linking $@ ..."
	$(Q)$(LINK.o) $^ $(LOADLIBES) $(LDLIBS) -o $@
	$(ECHO)
ifneq ($(MK_PRINT_EXE_SIZE),)
	$(Q)$(MK_PRINT_EXE_SIZE)
	$(ECHO)
endif

$(call MK_EXE_NAME, $(MK_LINK_TARGET)) : $(MK_OBJ_FILES)

endif
